package de.fitko.fitconnect.api.services;

import com.nimbusds.jose.jwk.RSAKey;
import de.fitko.fitconnect.api.domain.auth.OAuthToken;
import de.fitko.fitconnect.api.domain.model.metadata.Metadata;
import de.fitko.fitconnect.api.domain.model.metadata.attachment.Attachment;
import de.fitko.fitconnect.api.domain.model.metadata.data.Data;
import de.fitko.fitconnect.api.domain.model.submission.SubmissionRequest;
import de.fitko.fitconnect.api.domain.model.submission.SubmissionResponse;
import de.fitko.fitconnect.api.domain.validation.ValidationResult;

import java.io.File;
import java.util.List;
import java.util.Optional;
import java.util.UUID;

/**
 * A technical system that creates a submission via the FIT-Connect Submission API.
 * <p>
 * The Sender acts as a common interface wrapping all client functionality to authenticate <p>
 * and create a valid {@link SubmissionRequest} including encrypted {@link Data} and {@link Attachment}s
 *
 * @see <a href="https://docs.fitko.de/fit-connect/docs/sending/overview">Sending Submissions</a>
 */
public interface Sender {

    /**
     * Validates the integrity of the public key and its chain.
     *
     * @param publicKey to validate
     * @return {@link ValidationResult} with a list of errors if the validation failed
     */
    ValidationResult validatePublicKey(final RSAKey publicKey);

    /**
     * Encrypts the submission data payload (json or xml) with JWE (JSON-Web-Encryption).
     *
     * @param publicKey the public encryption key the data is encrypted with
     * @param data the {@link Data} payload that is encrypted
     * @return the JWE-encrypted {@link Data}, is empty if en error occurred
     */
    Optional<Data> encryptSubmissionData(final RSAKey publicKey, final Data data);

    /**
     * Encrypts the submission attachment binary data with JWE (JSON-Web-Encryption).
     *
     * @param publicKey the public encryption key the attachment is encrypted with
     * @param attachment the {@link Attachment} that is encrypted
     * @return the JWE-encrypted {@link Attachment}, is empty if en error occurred
     */
    Optional<Attachment> encryptAttachment(final RSAKey publicKey, final Attachment attachment);

    /**
     * Authenticates the {@link Sender} against the FitConnect-Auth-API and retrieves an {@link OAuthToken}.
     *
     * @param clientId a unique client identifier
     * @param clientSecret the applications secret key
     * @param scope 1 or more client scopes that determine if a submission is accepted by the client
     * @return {@link OAuthToken}, is empty if an error occurred
     */
    Optional<OAuthToken> retrieveOAuthToken(final String clientId, final String clientSecret, final String... scope);

    /**
     * Creates and uploads a {@link Metadata} object that contains {@link Data}, {@link Attachment} and their hashes so a subscriber
     * can verify the integrity.
     *
     * @param data {@link Data} object of the json or xml payload
     * @param attachments a list of binary {@link Attachment}s
     * @return a valid {@link Metadata} object with hashed {@link Data} and {@link Attachment}s, is empty if an error occurred
     */
    Optional<Metadata> createMetadata(final Data data, final List<Attachment> attachments);

    /**
     * Creates and uploads a {@link Metadata} object that contains {@link Data}, {@link Attachment} and their hashes so a subscriber
     * can verify the integrity.
     *
     * @param attachments a list of binary {@link Attachment}s
     * @return a valid {@link Metadata} object with hashed {@link Data} and {@link Attachment}s, is empty if an error occurred
     */
    Optional<Metadata> createMetadata(final List<Attachment> attachments);

    /**
     * Creates and announces a new {@link SubmissionRequest}
     *
     * @param submission with a destinationId, a list of attachmentIds and a serviceType
     * @return the created submission
     */
    Optional<SubmissionResponse> createSubmission(SubmissionRequest submission);


    /**
     * Posts the announced {@link SubmissionRequest}
     *
     * @param submissionId identifier of the announced submission
     * @param encryptedMetadata the encrypted metadata
     *
     * @return the submission
     */
    Optional<SubmissionResponse> sendSubmission(UUID submissionId, Metadata encryptedMetadata);


    void uploadAttachments(UUID submissionId, List<Attachment> encryptedAttachments);

    Optional<Attachment> createAttachment(File file);

    RSAKey getEncryptionKeyForDestination(UUID destinationId);
}
